function [meets_requirement, angles] = AttitudeCheck( x, AD )

%--------------------------------------------------------------------------
%   Michael Hauge
%   October 18, 2020
%
%   Given the state vector [r;v;q;w;b;z] and the attitude determination
%   system AD, this function returns a logical value as to whether the
%   spacecraft is pointing correctly as determined by the attitude
%   determination system given in the structure AD.
%
%--------------------------------------------------------------------------
%   Form:
%   meets_requirement = AttitudeCheck( x, AD )
%--------------------------------------------------------------------------
%
%   ------
%   Inputs 
%   ------
%   x           (14+n_rods,1) State vector [r;v;q;w;b;z]
%   AD          (1,1)   Data structure
%                   .half_angle     (1,1) Sensor FOV half-angle (deg)
%                   .elevation      (1,1) Sensor elevation angle (deg)
%               
%   -------
%   Outputs
%   -------
%   meets_requirement   (bool)  Whether or not the state of the CubeSat
%                               corresponds to the 'correct' orientation
%   angles              (1,4)   Angles, in radians, between the unit
%                               vectors of the IR sensors and nadir.
%
%--------------------------------------------------------------------------

% Unit vectors of IR sensors (pointing towards Earth) in body frame
el      = AD.elevation*pi/180; % sensor elevation in radians
z_comp  = sin(el);
xy_comp = cos(el);
u_body  = [[xy_comp;0;z_comp] [-xy_comp;0;z_comp] [0;xy_comp;z_comp] ...
            [0;-xy_comp;z_comp]];

% Quaternion from LVLH to body frame
qLVLHtoBody = QMult(QPose(QLVLH(x(1:3),x(4:6))), x(7:10));

% Unit vectors of IR sensors in LVLH frame
u_LVLH  = QTForm(qLVLHtoBody, u_body);

angles              = zeros(1,4);   % Initialize array
meets_requirement   = true;         % Output initially true

% Compute angle threshold
rE          = Constant('equatorial radius Earth');
h           = norm(x(1:3)) - rE; % height (km)
ang_size    = asin(rE/(rE + h)); % apparent size of Earth (rad)
threshold   = ang_size + AD.half_angle*pi/180; % angle threshold (rad)

for i = 1:4 % For each unit vector
    % Angle between unit vector and nadir ([0;0;1])
    angles(i) = acos(Dot(u_LVLH(:,i),[0;0;1]));
    
    % If the angle between unit vector and nadir is greater than rho
    % (apparent size) plus the sensor FOV, the requirement is not met
    if angles(i) > threshold
        meets_requirement = false;
    end
end

