function [xOut, t, simData] = CubeSatSimulation( sat, sim )

%--------------------------------------------------------------------------
%   Michael Hauge
%   November 23, 2020
%
%   Simulates the attitude dynamics of a CubeSat (the structure sat)
%   under the conditions given by the structure sim. Gives the time history
%   of the state of the CubeSat, xOut; the time data, t; and a structure
%   with information about the simulation, simData.
%
%--------------------------------------------------------------------------
%   Relies on the following scripts not included in the base PSS CubeSat
%   Control Toolbox, or modified forms of those included in the Toolbox:
%       RHSCubeSatHysteresis, CubeSatEnvironment, NorthDeviation,
%       AttitudeCheck, CubeSatRadiationPressure
%
%   In addition, the MATLAB app CubeSatADSSDesign.mlapp relies on:
%       PlotApp.m, IntegrateIMU.m
%--------------------------------------------------------------------------
%   Form:
%   [xOut, t, simData] = CubeSatSimulation( sat, sim )
%--------------------------------------------------------------------------
%
%   ------
%   Inputs (* = optional)
%   ------
%   sat     (1,1) Data structure
%               .size       (str)   CubeSat size
%               .satMass    (1,1)   CubeSat mass (kg)
%               .COM        (3,1)   COM location (m)
%               .dipole     (3,1)   Permanent dipole (ATM^2)
%               .rod_pairs  (1,1)   Number of hysteresis rod pairs
%               .rod_u      (3,2)   Unit vectors of hysteresis rods
%               .boom       (1,1)   Data structure (can be empty)
%                   *.end_mass      (1,1) Boom end mass (kg)
%                   *.length        (1,1) Extension from geometric center
%                                       (positive: nadir, negative: zenith)
%                   *.deploy_lat    (1,1) Latitude for boom deployment
%                   *.require_lat   (bool) Require latitude condition
%                   *.require_AD    (bool) Require AD verification
%                   *.AD_duration   (1,1) Length of required AD
%                                           verification (min)
%               .active     (1,1)   Data structure (can be empty)
%                   *.dipole        (3,1) Active dipole (torque rod)(ATM^2)
%                   *.trigger_lat   (1,1) Trigger lat (deg or 'initial')
%
%   sim     (1,1) Data structure
%               .jD0        (1,1)   Simulation start date (Julian date)
%               .w0         (3,1)   Initial tipoff rates (rad/s)
%               .el         (1,6)   Orbital elements [a i W w e M] (rad)
%               .numOrbits  (1,1)   Simulation duration (orbits)
%               .dT         (1,1)   Simulation time step (sec)
%               .aero_calc  (1,1)   Whether or not to compute drag/SRP
%                                   1 = never, 2 = after boom deployment,
%                                   3 = always
%               .radio_detumble  (1,1)  Rate for comms detumble (rad/s)
%               .att_detumble    (1,1)  Rate for attitude detumble (rad/s)
%               .att_dt_duration (1,1)  Length of required attitude
%                                           detumble verification (min)
%               .AD         (1,1)   Data structure
%                   .half_angle  (1,1)  IR sensor FOV half-angle (deg)
%                   .trigger     (1,1)  Deviation to trigger sensors (deg)
%                   .elevation   (1,1)  IR sensor elevation angle (deg)
%               
%   -------
%   Outputs
%   -------
%   xOut    (14+2*sat.rod_pairs,1)  Time history of state vector
%                                   [r, v, q, w, b, z]
%                                   r: ECI position of CubeSat (km)
%                                   v: ECI velocity of CubeSat (km/s)
%                                   q: CubeSat ECI to body quaternion
%                                   w: CubeSat body rates (rad/s)
%                                   b: CubeSat battery charge (J)
%                                   z: Magnetic flux density in hysteresis 
%                                      rods (T)
%   t           (1,:)   Time associated with each data point
%   simData     (1,1)   Data structure
%                   .pOrb   (1,1)   Orbital period (sec)
%                   .inr    (3,3)   Final moment of inertia tensor (kg*m^2)
%                   .lla    (3,:)   [lat (deg); long (deg); alt (km)]
%                   .eul    (3,:)   Euler angles (rad)
%                   .alpha  (3,:)   Body accelerations (rad/s^2)
%                   .nadir  (1,:)   Deviation from nadir (rad)
%                   .north  (1,:)   Deviation from magnetic north (rad)
%                   .qAnim  (4,:)   Quaternion for animation
%                   .bFieldBody (3,:)   Magnetic field in body frame (T)
%                   .bFieldLVLH (3,:)   Magnetic field in LVLH frame (T)
%                   .uSunBody   (3,:)   Sun unit vector in body frame
%                   .torquerod_time (1,2)   Torque rod [on,off] indices
%                   .boom_deploy    (1,1)   Boom deployment index
%                   .radio_DT_time  (1,1)   Index when detumbled (radio)
%                   .att_DT_time    (1,1)   Index when detumbled (attitude)
%                   .dist   (1,1)   Data structure
%                       .fAero  (3,:)   Aerodynamic forces (N)
%                       .tAero  (3,:)   Aerodynamic torques (Nm)
%                       .fOpt   (3,:)   Optical forces (N)
%                       .tOpt   (3,:)   Optical torques (Nm)
%                       .tMag   (3,:)   Magnetic torques (Nm)
%                       .tHyst  (3,:)   Magnetic hysteresis torques (Nm)
%                       .rodH   (2*sat.rod_pairs,:) Magnetic field strength
%                                       in hysteresis rods (A/m)
%                       .tGG    (3,:)   Gravity gradient torques (Nm)
%                   .fourier    (1,1) Data structure
%                       .f          (1,:)   Frequency domain (Hz)
%                       .p_roll     (1,:)   Amp spectrum of roll data
%                       .p_pitch    (1,:)   Amp spectrum of pitch data
%                       .p_yaw      (1,:)   Amp spectrum of yaw data
%
%--------------------------------------------------------------------------

%% Initialize data structures ---------------------------------------------
% Initialize simData output
simData = struct;

%--------------------- Define CubeSat data structure ---------------------%
% Size and mass
cs          = RHSCubeSatHysteresis;
cs.size     = sat.size;
mSat        = sat.satMass;
Isat        = InertiaCubeSat(cs.size, mSat);
rCM         = sat.COM;
sat_only    = MassStructure(mSat, Isat, rCM);

boom_deployed = false;
density     = 8500; % kg/m^3; density of C36000 brass

if ~isempty(sat.boom) % If there is a gravity boom
    % Check if gravity boom is initially deployed
    if ~sat.boom.require_lat && ~sat.boom.require_AD
        boom_deployed       = true;
        simData.boom_deploy = 1; % Boom deploys at index 1
        
        mGG         = sat.boom.end_mass;
        rGG         = [0;0;sat.boom.length/100];
        Iboom       = -mGG*SkewSq(rGG);
        boom        = MassStructure(mGG, Iboom, rGG);
        mass        = AddMass([sat_only boom]);
        cs.mass     = mass.mass;
        cs.inertia  = mass.inertia;
        cs.cM       = mass.cM;
    else
        cs.mass     = sat_only.mass;
        cs.inertia  = sat_only.inertia;
        cs.cM       = sat_only.cM;
        AD_persistence = ceil(sat.boom.AD_duration*60/sim.dT);
        simData.boom_deploy = -1; % Value for boom not yet deployed
    end
else % If there is no gravity boom
    cs.mass     = sat_only.mass;
    cs.inertia  = sat_only.inertia;
    cs.cM       = sat_only.cM;
    
    simData.boom_deploy = []; % there is no boom deployment index
end

%----- Aerodynamics calculations -----%
C_D = 2.7; % Coefficient of drag

% If aerodynamics are never calculated
if sim.aero_calc == 1
    cs.surfData = []; % ignore aerodynamic drag
    
% If aerodynamics are only calculated after boom deployment
elseif sim.aero_calc == 2
    if boom_deployed
        % Define CubeSat Surfaces
        surfD       = struct;
        surfD.cD    = C_D;
        surfD.cM    = cs.cM;
        [a,n,r]     = CubeSatFaces(cs.size,1);
        surfD.area        = a;
        surfD.nFace       = n;
        surfD.rFace       = r;
        surfD.att.type    = 'eci';
        
        side        = (mGG/density)^(1/3); % m; side length of end mass
        boom_areas  = side^2*ones(1,6);
        boom_n      = n;
        boom_r      = rGG*ones(1,6);
        
        surfD.area  = [surfD.area boom_areas];
        surfD.nFace = [surfD.nFace boom_n];
        surfD.rFace = [surfD.rFace boom_r];
        
        cs.surfData = surfD;
        cs.surfData = CubeSatRadiationPressure(cs.surfData); % add needed 
                                                    % fields to cs.surfData
    else
        cs.surfData = [];
    end
    
% If aerodynamics will always be calculated
elseif sim.aero_calc == 3
    % Define CubeSat Surfaces
    surfD       = struct;
    surfD.cD    = C_D;
    surfD.cM    = cs.cM;
    [a,n,r]     = CubeSatFaces(cs.size,1);
    surfD.area        = a;
    surfD.nFace       = n;
    surfD.rFace       = r;
    surfD.att.type    = 'eci';
    
    if boom_deployed % Add surfaces of gravity boom to model
        side        = (mGG/density)^(1/3); % m; side length of end mass
        boom_areas  = side^2*ones(1,6);
        boom_n      = n;
        boom_r      = rGG*ones(1,6);
        
        surfD.area  = [surfD.area boom_areas];
        surfD.nFace = [surfD.nFace boom_n];
        surfD.rFace = [surfD.rFace boom_r];
    end
    
    cs.surfData = surfD;
    cs.surfData = CubeSatRadiationPressure(cs.surfData); % add needed 
                                                    % fields to cs.surfData
end

% Initial dipole
torque_rod_on = false;
if ~isempty(sat.active) % If there is an active magnetic system
    % If torque rod is initially on
    if strcmp(sat.active.trigger_lat, 'initial')
        cs.dipole = sat.dipole + sat.active.dipole;
        torque_rod_on = true;
        simData.torquerod_time = [1,-1];
    else
        simData.torquerod_time = [-1,-1];
        cs.dipole = sat.dipole;
    end
else % If there is no active magnetic system
    cs.dipole = sat.dipole;
    % There are also no on/off indices for the torque rod
    simData.torquerod_time = []; 
end

% Hysteresis subsystem
if sat.rod_pairs > 0
    cs.dampingData.u = sat.rod_u;
    for i = 2:sat.rod_pairs
        cs.dampingData.u = [cs.dampingData.u sat.rod_u];
    end
    cs.dampingData.v = pi*(0.001/2)^2*0.095*ones(1,size(cs.dampingData.u,2));
else
    cs.dampingData.u = [];
    cs.dampingData.v = [];
end

% Julian date
cs.jD0  = sim.jD0;


%% Set up initial state vector --------------------------------------------
a    = sim.el(1);
pOrb = Period(a);

[r0,v0] = El2RV(sim.el);        % Initial position and velocity
q0   = QLVLH(r0, v0);           % Initial ECI to body quaternion
n    = [0; -OrbRate(a); 0];     % Rotation at orbit rate (LVLH wrt ECI)
w0   = sim.w0 + n;              % Initial body rates
b0   = 0;                       % Battery charge
[bI,bIDot] = BDipole(r0, cs.jD0, v0);

if sat.rod_pairs > 0
    uECI = QTForm(q0, cs.dampingData.u);
    mu0  = 4e-7*pi;
    hMag = Dot(uECI, bI)/mu0;
    hMagDot = Dot(uECI, bIDot)/mu0;
    % Magnetic flux density in hysteresis rods
    z0   = BFromHHysteresis(hMag, hMagDot, cs.dampingData);  
else
    z0 = [];
end

% Initial state vector
x       = [r0; v0; q0; w0; b0; z0'];

% Other initial values
lla0rad = CoordinateTransform('ECI', 'LLR', x(1:3), cs.jD0);
lla0    = [lla0rad(1)*180/pi; lla0rad(2)*180/pi; lla0rad(3)]; 
qLVLHtoBody0 = QMult(QPose(q0), q0);       % Initial quaternion
eul0    = Q2Eul(QMult(QPose(q0), q0));     % Initial Euler angles
nd0     = acos(cos(eul0(1))*cos(eul0(2))); % Initial nadir pointing error
bField0 = QForm(q0, BDipole(r0, cs.jD0, v0));  % Initial magnetic field
nr0     = NorthDeviation(r0,q0,cs.jD0);    % Initial north pointing error


%% Simulation parameters and setup ----------------------------------------
orbits  = sim.numOrbits;
tEnd    = pOrb*orbits;
dT      = sim.dT;
nSim    = floor(tEnd/dT);

% Logical values
att_detumbled   = false;
radio_detumbled = false;

% Set detumbled times to -1 initially to detect if sat is never detumbled
simData.radio_DT_time   = -1;
simData.att_DT_time     = -1;

% Check if initially detumbled
% (If initially detumbled, the check for persistence is bypassed)
w_tot0_rad = sqrt(w0(1)^2 + w0(2)^2 + w0(3)^2);
w_tot0_att = sqrt(w0(1)^2 + w0(2)^2); % att detumble doesn't care about yaw
if w_tot0_rad < sim.radio_detumble
    radio_detumbled = true;
    simData.radio_DT_time = 1;
end
if w_tot0_att < sim.att_detumble
    att_detumbled = true;
    simData.att_DT_time = 1;
end

persistence_counter = 0;    % Counter for AD persistence condition
att_counter         = 0;    % Counter for attitude detumble persistence

att_persistence = ceil(sim.att_dt_duration*60/sim.dT);


%% Initialize arrays ------------------------------------------------------
% Initial step
[xDot, distur] = RHSCubeSatHysteresis(x, 0, cs);

% Time series data
xOut    = [x zeros(size(x,1),nSim)];    % State vector
lla     = [lla0 zeros(3,nSim)];         % LLA position (angles in rad)
eul     = [eul0 zeros(3,nSim)];         % Euler angles (rad)
alpha   = [xDot(11:13) zeros(3,nSim)];  % Body accelerations (rad/s^2)
nadir   = [nd0 zeros(1,nSim)];          % Nadir pointing error (rad)
north   = [nr0 zeros(1,nSim)];          % North pointing error (rad)
qAnim   = [qLVLHtoBody0 zeros(4,nSim)]; % Quaternion for animation
bFieldBody  = [bField0 zeros(3,nSim)];  % Magnetic field in body frame (T)
bFieldLVLH  = [bField0 zeros(3,nSim)];  % Magnetic field in LVLH frame (T)
fAero   = [distur.fAerodyn zeros(3,nSim)];  % Drag forces (N)
tAero   = [distur.tAerodyn zeros(3,nSim)];  % Aerodynamic torque (Nm)
fOpt    = [distur.fOptical zeros(3,nSim)];  % Optical force (N)
tOpt    = [distur.tOptical zeros(3,nSim)];  % Optical torque (Nm)
tMag    = [distur.tMag zeros(3,nSim)];      % Magnetic torque (Nm)
tHyst   = [distur.tHyst zeros(3,nSim)];     % Damping torque (Nm)
tGG     = [distur.tGG zeros(3,nSim)];       % Gravity gradient torque (Nm)
rodH    = [distur.rodH zeros(2*sat.rod_pairs,nSim)]; % Magnetic field  
                                        % strength in hysteresis rods (A/m)
uSunBody = [distur.uSunBody zeros(3,nSim)]; % Sun unit vector in body frame


%% Run simulation ---------------------------------------------------------
t = 0;
wb = waitbar(0,'CubeSat Simulation');
for k = 1:nSim
    % Calculate latitude
    lla_pos  = CoordinateTransform('ECI', 'LLR', x(1:3), cs.jD0+t/86400);
    lla(:,k+1)  = [lla_pos(1)*180/pi; lla_pos(2)*180/pi; lla_pos(3)];
    latitude    = lla_pos(1)*180/pi; % latitude in degrees
    
    % Calculate instantaneous angular velocity in rad/s
    w           = x(11:13);                        % Body angular rates
    w_tot_rad   = sqrt(w(1)^2 + w(2)^2 + w(3)^2);  % Total w (rad/s)
    w_tot_att   = sqrt(w(1)^2 + w(2)^2);           % Doesn't care about yaw
    
    % Compare to radio detumble threshold
    if ~radio_detumbled && w_tot_rad < sim.radio_detumble
        radio_detumbled = true;
        simData.radio_DT_time = k+1;
    end
    
    % Compare to attitude detumble threshold
    if ~att_detumbled
        if w_tot_att < sim.att_detumble
            att_counter = att_counter + 1;
        else
            att_counter = 0;
        end
        
        if att_counter >= att_persistence
            att_detumbled = true;
            simData.att_DT_time = k+1;
        end
    end
    
    %--------------- Logic for torque rod and gravity boom ---------------%   
    % If there is a boom and it is not already deployed, and the satellite
    % is detumbled
    if ~isempty(sat.boom) && ~boom_deployed && att_detumbled
        % If an active system is present
        if ~isempty(sat.active)
            % If the torque rod is not already on
            if ~torque_rod_on
                % If the latitude trigger is met
                if (abs(latitude) > abs(sat.active.trigger_lat))
                    % Turn the torque rod on
                    cs.dipole       = sat.dipole + sat.active.dipole;
                    
                    torque_rod_on   = true;
                    simData.torquerod_time = [k+1,-1]; % 'on' index
                end
            else % If the torque rod is already on
                % Check latitude and attitude triggers for boom deployment
                lat_check = (abs(latitude) > abs(sat.boom.deploy_lat));
                att_check = AttitudeCheck(x, sim.AD);
                
                if att_check
                    persistence_counter = persistence_counter + 1;
                else
                    persistence_counter = 0;
                end
                
                % If latitude check required and is met, or else latitude
                % check is not required
                if (sat.boom.require_lat && lat_check) || ~sat.boom.require_lat
                    % If AD check required and is met, or isn't required
                    if sat.boom.require_AD && att_check...
                            && persistence_counter >= AD_persistence ||...
                            ~sat.boom.require_AD
                        %--------- Deploy the boom ---------%
                        boom_deployed = true;
                        % Index of boom deployment
                        simData.boom_deploy = k+1; 
                        
                        % Update the mass and inertia
                        mGG         = sat.boom.end_mass;
                        rGG         = [0;0;sat.boom.length/100];
                        Iboom       = -mGG*SkewSq(rGG);
                        boom        = MassStructure(mGG, Iboom, rGG);
                        mass        = AddMass([sat_only boom]);
                        cs.mass     = mass.mass;
                        cs.inertia  = mass.inertia;
                        cs.cM       = mass.cM;
                        
                        % Update cs.surfData if necessary
                        if sim.aero_calc == 2
                            % Create all CubeSat and boom surfaces
                            cs.surfData.cD  = C_D;
                            cs.surfData.cM  = cs.cM;
                            [a,n,r]         = CubeSatFaces(cs.size,1);
                            
                            side        = (mGG/density)^(1/3); % m; 
                                           % side length of end mass (cube)
                            boom_areas  = side^2*ones(1,6);
                            boom_n      = n;
                            boom_r      = rGG*ones(1,6);
        
                            cs.surfData.area  = [a boom_areas];
                            cs.surfData.nFace = [n boom_n];
                            cs.surfData.rFace = [r boom_r];
                            cs.surfData.att.type = 'eci';
                        
                            cs.surfData = CubeSatRadiationPressure(cs.surfData);
                        elseif sim.aero_calc == 3
                            [a,n,r]         = CubeSatFaces(cs.size,1);
                            % Add boom surfaces to cs.surfData
                            side        = (mGG/density)^(1/3); % m; 
                                           % side length of end mass (cube)
                            boom_areas  = side^2*ones(1,6);
                            boom_n      = n;
                            boom_r      = rGG*ones(1,6);
        
                            cs.surfData.area  = [cs.surfData.area boom_areas];
                            cs.surfData.nFace = [cs.surfData.nFace boom_n];
                            cs.surfData.rFace = [cs.surfData.rFace boom_r];
                        
                            cs.surfData = CubeSatRadiationPressure(cs.surfData);
                        end
                        
                        %----- Turn off the torque rod -----%
                        cs.dipole       = sat.dipole;
                        torque_rod_on   = false;
                        simData.torquerod_time(2) = k+1; % 'off' index
                    end
                end
            end
        else % If no active system is present
            % Check latitude and attitude triggers
            lat_check   = (abs(latitude) > abs(sat.boom.deploy_lat));
            att_check   = AttitudeCheck(x, sim.AD);
            
            if att_check
                persistence_counter = persistence_counter + 1;
            else
                persistence_counter = 0;
            end
            
            % If latitude check required and is met, or else latitude check
            % is not required
            if (sat.boom.require_lat && lat_check) || ~sat.boom.require_lat
                % If AD check required and is met, or isn't required
                if sat.boom.require_AD && att_check...
                        && persistence_counter >= AD_persistence ||...
                        ~sat.boom.require_AD
                    %--------- Deploy the boom ---------%
                    boom_deployed       = true;
                    simData.boom_deploy = k+1; % index of boom deployment
                    
                    % Update the mass and inertia
                    mGG         = sat.boom.end_mass;
                    rGG         = [0;0;sat.boom.length/100];
                    Iboom       = -mGG*SkewSq(rGG);
                    boom        = MassStructure(mGG, Iboom, rGG);
                    mass        = AddMass([sat_only boom]);
                    cs.mass     = mass.mass;
                    cs.inertia  = mass.inertia;
                    cs.cM       = mass.cM;
                    
                    % Update cs.surfData if necessary
                    if sim.aero_calc == 2
                        % Create all CubeSat and boom surfaces
                        cs.surfData.cD  = C_D;
                        cs.surfData.cM  = cs.cM;
                        [a,n,r]         = CubeSatFaces(cs.size,1);
                        
                        side        = (mGG/density)^(1/3); % m; 
                                           % side length of end mass (cube)
                        boom_areas  = side^2*ones(1,6);
                        boom_n      = n;
                        boom_r      = rGG*ones(1,6);
    
                        cs.surfData.area  = [a boom_areas];
                        cs.surfData.nFace = [n boom_n];
                        cs.surfData.rFace = [r boom_r];
                        cs.surfData.att.type = 'eci';
                    
                        cs.surfData = CubeSatRadiationPressure(cs.surfData);
                    elseif sim.aero_calc == 3
                        [a,n,r]         = CubeSatFaces(cs.size,1);
                        % Add boom surfaces to cs.surfData
                        side        = (mGG/density)^(1/3); % m; 
                                           % side length of end mass (cube)
                        boom_areas  = side^2*ones(1,6);
                        boom_n      = n;
                        boom_r      = rGG*ones(1,6);
        
                        cs.surfData.area  = [cs.surfData.area boom_areas];
                        cs.surfData.nFace = [cs.surfData.nFace boom_n];
                        cs.surfData.rFace = [cs.surfData.rFace boom_r];
                        
                        cs.surfData = CubeSatRadiationPressure(cs.surfData);
                    end
                end
            end
        end
    end
    
    %-------------------- Proceed with the simulation --------------------%
    
    % Calculate magnetic field
    bFieldBody(:,k+1) = QForm(x(7:10), BDipole(x(1:3), cs.jD0+t/86400, x(4:6)));
    
    % Time step with 4th-order Runge-Kutta
    x = RK4(@RHSCubeSatHysteresis, x, dT, t, cs);
    
    % Normalize quaternion
    x(7:10) = x(7:10)/norm(x(7:10));
    
    % Unpack disturbance data for this time step
    [xT, dist]      = RHSCubeSatHysteresis(x, t, cs);
    fAero(:,k+1)    = dist.fAerodyn;
    tAero(:,k+1)    = dist.tAerodyn;
    fOpt(:,k+1)     = dist.fOptical;
    tOpt(:,k+1)     = dist.tOptical;
    tMag(:,k+1)     = dist.tMag;
    tHyst(:,k+1)    = dist.tHyst;
    tGG(:,k+1)      = dist.tGG;
    rodH(:,k+1)     = dist.rodH;
    uSunBody(:,k+1) = dist.uSunBody;
    
    % Quaternion and Euler angles
    qLVLHtoBody     = QMult(QPose(QLVLH(x(1:3),x(4:6))), x(7:10));
    eul(:,k+1)      = Q2Eul(qLVLHtoBody);
    alpha(:,k+1)    = xT(11:13);
    qAnim(:,k+1)    = qLVLHtoBody;
    
    bFieldLVLH(:,k+1) = QTForm(qLVLHtoBody, bFieldBody(:,k+1));
    
    % Nadir and magnetic north deviation
    nadir(k+1)      = acos(cos(eul(1,k+1))*cos(eul(2,k+1)));
    north(k+1)      = NorthDeviation(x(1:3),x(7:10),cs.jD0+t/86400);
    
    % Update time and time history of state vector
    xOut(:,k+1)     = x;
    t               = t + dT;
    
    waitbar(k/nSim, wb);
    
end
close(wb);


%% Data processing --------------------------------------------------------
%----- Fourier series of body angular rates -----%
L       = nSim + 1; % length of signal
Fs      = 1/dT;     % sampling frequency

% Roll, pitch, yaw data
roll_rates  = xOut(11,:);
pitch_rates = xOut(12,:);
yaw_rates   = xOut(13,:);

% Fourier transforms
roll_F  = fft(roll_rates);
pitch_F = fft(pitch_rates);
yaw_F   = fft(yaw_rates);

% Compute spectra
P2_roll = abs(roll_F/L);
P1_roll = P2_roll(1:floor(L/2+1));
P1_roll(2:end-1) = 2*P1_roll(2:end-1);

P2_pitch = abs(pitch_F/L);
P1_pitch = P2_pitch(1:floor(L/2+1));
P1_pitch(2:end-1) = 2*P1_pitch(2:end-1);

P2_yaw = abs(yaw_F/L);
P1_yaw = P2_yaw(1:floor(L/2+1));
P1_yaw(2:end-1) = 2*P1_yaw(2:end-1);

% Compute frequency domain
f = Fs*(0:(L/2))/L;

% Collect and assign outputs
simData.fourier         = struct;
simData.fourier.f       = f(2:end);
simData.fourier.p_roll  = P1_roll(2:end);
simData.fourier.p_pitch = P1_pitch(2:end);
simData.fourier.p_yaw   = P1_yaw(2:end);


%% Assign outputs ---------------------------------------------------------
% xOut has already been assigned (populated within simulation loop)
% simData.torquerod_time, .boom_deploy, .radio_DT_time, and .att_DT_time
%   have already been assigned

t = (0:nSim)*dT;

simData.pOrb    = pOrb;
simData.inr     = cs.inertia;
simData.lla     = lla;
simData.eul     = eul;
simData.alpha   = alpha;
simData.nadir   = nadir;
simData.north   = north;
simData.qAnim   = qAnim;
simData.bFieldBody = bFieldBody;
simData.bFieldLVLH = bFieldLVLH;
simData.uSunBody   = uSunBody;

simData.dist = struct;
simData.dist.fAero  = fAero;
simData.dist.tAero  = tAero;
simData.dist.fOpt   = fOpt;
simData.dist.tOpt   = tOpt;
simData.dist.tMag   = tMag;
simData.dist.tHyst  = tHyst;
simData.dist.tGG    = tGG;
simData.dist.rodH   = rodH;
