function [q, eul, nd, t_int] = IntegrateIMU( x, t, sim, simData )

%--------------------------------------------------------------------------
%   Michael Hauge
%   November 13, 2020
%
%   Given a time series of angular velocities, the initial quaternion, and
%   information about the simulation and its results, this function 
%   returns the reconstructed time series of the quaternion, Euler 
%   angles, and nadir deviation.
%
%--------------------------------------------------------------------------
%   Form:
%   [q, eul, nd, t_int] = IntegrateIMU( x, sim, simData )
%--------------------------------------------------------------------------
%
%   ------
%   Inputs 
%   ------
%   x           (13,n) Time history of the state of the simulation
%                       [r; v; q (ECI to body); w (in body frame)]
%   t           (1,n)  Time series for simulation
%   sim         (1,1)  Data structure
%                   .dT     (1,1)   Simulation time step (sec)
%                   .AD     (1,1)   Data structure
%                               .imu_pd (1,1) IMU sample period (sec)
%   simData     (1,1)  Data structure
%                   .eul    (3,n)   Euler angles (rad)
%               
%   -------
%   Outputs
%   -------
%   q           (4,n) Reconstructed time series of quaternion from ECI
%                       frame to body frame
%   eul         (3,n) Reconstructed time series of Euler angles (rad)
%   nd          (1,n) Reconstructed time series of nadir deviation (rad)
%   t_int       (1,n) Time series for outputs
%
%--------------------------------------------------------------------------

% Extract the data
w_sim  = x(11:13,:);
r_sim  = x(1:3,:);
v_sim  = x(4:6,:);
q0     = x(7:10,1);
eul0   = simData.eul(:,1);
nd0    = acos(cos(eul0(1))*cos(eul0(2)));

% Choose the multiple of the time step
step_mult = floor(sim.AD.imu_pd/sim.dT);
dt = sim.dT*step_mult;

%------ Get data to be integrated (select from certain time steps) ------%
num_step = floor(length(t)/step_mult);

% Preallocate for speed
w       = zeros(3,num_step);
r       = zeros(3,num_step);
v       = zeros(3,num_step);
t_int   = zeros(1,num_step);

% Select from simulation data
for i = 1:num_step
    w(:,i) = w_sim(:,(i-1)*step_mult+1);
    r(:,i) = r_sim(:,(i-1)*step_mult+1);
    v(:,i) = v_sim(:,(i-1)*step_mult+1);
    t_int(i) = t((i-1)*step_mult+1);
end

%---------------- Apply the direct multiplication method ----------------%
% Preallocate for speed
q_DM    = [q0 zeros(4,num_step-1)];
eul_DM  = [eul0 zeros(3,num_step-1)];
nd_DM   = [nd0 zeros(1,num_step-1)];

% Integrate
for i = 1:size(q_DM,2)-1
    % Magnitude of angular velocity
    w_mag = norm(w(:,i));
    
    % Construct the quaternion for multiplication
    q_prime = [cos(w_mag*dt/2); w(:,i)/w_mag*sin(w_mag*dt/2)];
    
    % Propagate the quaternion from ECI frame to body frame
    q_DM(:,i+1) = QMult(q_DM(:,i),QPose(q_prime));
    
    % Find the Euler angles
    qLVLHtoBody     = QMult(QPose(QLVLH(r(:,i+1),v(:,i+1))), q_DM(:,i+1));
    eul_DM(:,i+1)   = Q2Eul(qLVLHtoBody);
    
    % Find the deviation from nadir
    nd_DM(i+1) = acos(cos(eul_DM(1,i+1))*cos(eul_DM(2,i+1)));
end

% Assign outputs
q   = q_DM;
eul = eul_DM;
nd  = nd_DM;

end
