function deviation = NorthDeviation( r, q, jD )

%--------------------------------------------------------------------------
%   Michael Hauge
%   October 16, 2020
%
%   Given the ECI position and the ECI to body quaternion of a CubeSat, as
%   well as the Julian date, this function computes the angle, in radians, 
%   between the +X direction in the CubeSat body frame and the direction of
%   magnetic north as defined by the location of the geomagnetic north
%   pole.
%
%--------------------------------------------------------------------------
%   Form:
%   deviation = NorthDeviation( r, q, jD )
%--------------------------------------------------------------------------
%
%   ------
%   Inputs 
%   ------
%   r           (3,1)   ECI position vector of the CubeSat
%   q           (4,1)   CubeSat ECI to body quaternion
%   jD          (1,1)   Julian date
%               
%   -------
%   Outputs
%   -------
%   deviation   (1,1)   Angle in radians between the +X direction in the
%                       CubeSat body frame and local magnetic north
%
%--------------------------------------------------------------------------

np_lat  = 80.65*pi/180;            % Latitude of geomagnetic north pole
np_long = -72.68*pi/180;           % Longitude of geomagnetic north pole
rP      = Constant('equatorial radius Earth');
np_LLA  = [np_lat; np_long; rP];   % LLA position of geomagnetic north pole

% ECI position of geomagnetic north pole
np_ECI  = CoordinateTransform('LLR','ECI',np_LLA,jD);

% Vector from CubeSat to geomagnetic north pole
r_CStoGM = np_ECI - r;

% Project onto the plane normal to r (tangent to Earth's surface)
n       = r/norm(r);                    % Unit normal to plane
r_tan   = r_CStoGM - Dot(r_CStoGM,n)*n; % Projection in tangent plane

% Get the local magnetic north direction in the body frame
mag_north = QForm(q, r_tan);

% Get the angle between the +X direction and local magnetic north
unit_x      = [1;0;0];
deviation   = acos(Dot(mag_north, unit_x)/norm(mag_north));




