function [xDot,dist,power] = RHSCubeSatHysteresis( x, t, d )

%% Right-hand-side for a CubeSat orbit and attitude dynamical model.
%
% Michael Hauge
% November 29, 2020
%
% Based on RHSCubeSat.m, Princeton Satellite Systems (c) 2009, 2016
%
% Includes drag and optical surface disturbances, magnetic dipole and 
% hysteresis, gravity gradient, rigid body dynamics and power. Uses the
% scale height atmosphere model and applies a point-mass gravity model. 
% Computes sun location with SunV1 and accounts for eclipses from the 
% Earth. The surface parameters in surfData are passed to both the optical 
% and aerodynamic disturbance model functions.
%
% The states are:
% [position;velocity;quaternion;angular velocity;battery charge;rod fields]
% The battery charge units are J. If there are wheel states they must be 
% between the spacecraft angular velocity and the battery charge, and the 
% indices be logged in kWheels.
%
% Can output the disturbance forces and torques for post-processing.
% There is also a call to retrieve the default data structure.
%--------------------------------------------------------------------------
%   Form:
%   [xDot,dist,power] = RHSCubeSatHysteresis( x, t, d )
%   d = RHSCubeSatHysteresis
%--------------------------------------------------------------------------
%
%   ------
%   Inputs
%   ------
%   x        (14,1)    [r;v;q;w;wRWA;b;z]
%   t         (1,1)    Time, sec
%   d          (.)     Data structure
%                      .jD0      (1,1) Julian date of epoch
%                      .mass     (1,1) Spacecraft mass (kg)
%                      .inertia  (3,3) Inertia matrix (kg-m2)
%                      .dipole   (3,1) Residual dipole (ATM^2)
%                      .power     (.)  Power data, see SolarCellPower 
%                      .aeroModel    * Handle, see CubeSatAero
%                      .opticalModel * Handle, see CubeSatRadiationPressure
%                      .surfData  (.)  optional; empty to skip drag/optical calcs
%                                      .cD    (1,1) Drag coefficient
%                                      .cM    (3,1) Center of mass (m)
%                                      .area  (1,n) Area (m2)
%                                      .nFace (3,n) Face normals
%                                      .rFace (3,n) Face locations (m)
%                                      .att    (.)  Attitude model
%                                      .sigma (3,n) Optical coefficients
%                                      .planet (1)  Planet effects flag
%                      .atm       (.) optional; empty to skip J70 and use AtmDens2
%                      .kWheels         (n), empty if no wheels, indices of wRWA
%                      .inertiaRWA     (1,1), optional, polar inertia (kg-m2)
%                      .tRWA           (3,1), optional, wheel torque (Nm)
%                      .dampingFun * Handle, see RHSHysteresisDamper
%                      .dampingData    Sub-structure for hysteresis damping
%                                      .u (3,n)     Rod unit vectors
%                                      .v (1,n)     Rod volumes (m3)
%                                      .Br (1,1)    Remanence (T)
%                                      .Bs (1,1)    Saturation induction (T)
%                                      .Hc (1,1)    Coercive force (A/m)
%
%   -------
%   Outputs
%   -------
%   x           (14,1)	   d[r;v;q;w;b;z]/dt
%   dist         (.)       Disturbances data
%    .fTotal  	   (3,1)   Total force, ECI frame  (N)
%    .tTotal       (3,1)   Total torque, body frame (Nm)
%    .fAerodyn	   (3,1)   Aerodynamic force
%    .tAerodyn     (3,1)   Aerodynamic torque
%    .fOptical	   (3,1)   Optical force
%    .tOptical     (3,1)   Optical torque
%    .tMag         (3,1)   Magnetic torque
%    .tGG          (3,1)   Gravity gradient torque
%    .tHyst        (3,1)   Magnetic hysteresis torque
%    .uSunBody     (3,1)   Sun unit vector in body frame
%   power       (1,1)      Power from solar cells (W)
%   
% 
%--------------------------------------------------------------------------
% See also SolarCellPower, CubeSatAero, CubeSatRadiationPressure,
% SolarFluxPrediction, CubeSatEnvironment, CubeSatSimulation,
% CubeSatRWASimulation
%--------------------------------------------------------------------------

if nargin == 0
  xDot = DefaultStruct;
  return;
end

% Get the states
%---------------
r      = x(1:3);
v      = x(4:6);
q      = x(7:10);
w      = x(11:13);
num_rods = size(d.dampingData.u,2);
b      = x(end-num_rods);
z      = x(end-num_rods+1:end);
xDot   = zeros(size(x));

% Environment data
%-----------------
d.atm = []; % Use atmosphere model based on scale heights (eliminate to use
                                                           % Jacchia model)
s = CubeSatEnvironment( x, t, d );

% Total power in - power out
%---------------------------
uSunBody       = QForm( q, s.uSun );
pSun           = s.nEcl*s.solarFlux*uSunBody;
power          = SolarCellPower( d.power, pSun );
p              = power - d.power.consumption;

% Limit the battery charging
%---------------------------
if( b >= d.power.batteryCapacity )
  if( p > 0 )
    p = 0;
  end
elseif( b <= 0 )
  if( p < 0 )
    p = 0;
  end
end

% Magnetic field
%---------------
bField = QForm( q, s.bField );
tMag   = Cross( d.dipole, bField );

% Gravity gradient
%-----------------
tGG    = GravityGradientFromR( q, d.inertia, r, s.mu );

% Magnetic hysteresis damping
%-----------------
hyst_state = [r;v;q;w;z];
[zDot,h]    = feval(d.dampingFun, hyst_state, s.bField, s.bFieldDot, d.dampingData);

% Add the drag/solar force and torque
%------------------------------------
fAerodyn = [0;0;0];
tAerodyn = [0;0;0];
fOptical = [0;0;0];
tOptical = [0;0;0];
if( ~isempty(d.surfData) )
  %d.surfData.att.qECIToBody = q;
	[fAerodyn, tAerodyn]    = feval( d.aeroModel, s, d.surfData );
    [fOptical, tOptical]    = feval( d.opticalModel, s, d.surfData );
end

tTotal = tGG + tMag + tAerodyn + tOptical + h.torqueDamper;
fTotal = fOptical + fAerodyn;

% Ideal orbital dynamics
%-----------------------
vDot   = fTotal*1e-3/d.mass - s.mu*r/Mag(r)^3;

% Dynamical equations
%--------------------
if ~isempty(d.kWheels)
  c      = x(d.kWheels); % Reaction wheel rates
  hT     = d.inertia*w + d.inertiaRWA*(c + w);
  wDot   = d.inertia\(tTotal - d.tRWA + Cross(w,hT));
  cDot   = d.tRWA/d.inertiaRWA - wDot;
  xDot(d.kWheels) = cDot;
else
  wDot   = d.inertia\(tTotal - Cross(w,d.inertia*w));
end

% RHS
%----
xDot(1:13) = [v;vDot;QIToBDot( q, w );wDot];
xDot(end-num_rods) = p;
xDot(end-num_rods+1:end) = zDot;

if nargout > 1
  dist.fOptical = fOptical;
  dist.tOptical = tOptical;
  dist.fAerodyn = fAerodyn;
  dist.tAerodyn = tAerodyn;
  dist.tGG      = tGG;
  dist.tMag     = tMag;
  dist.tHyst    = h.torqueDamper;
  dist.rodH     = h.hMag;
  dist.fTotal   = fTotal;
  dist.tTotal   = tTotal;
  dist.rho      = s.rho;
  dist.bField   = bField;
  dist.uSunBody = uSunBody;
end

%--------------------------------------------------------------------------
% Default data structure
%--------------------------------------------------------------------------
function d = DefaultStruct

d                       = struct;
d.jD0                   = Date2JD(2010);
d.mass                  = 1;
d.inertia               = 0.0016667*eye;
d.dipole                = [0;0;0];
d.power                 = SolarCellPower;
d.power.consumption     = 0.5;
d.power.batteryCapacity = 100;
d.surfData              = CubeSatAero;
d.surfData.att.type     = 'eci'; % for simulating, always use ECI quaternion
d.aeroModel             = @CubeSatAero;
d.opticalModel          = @CubeSatRadiationPressure;
d.surfData              = CubeSatRadiationPressure(d.surfData);
[aP, f, fHat, fHat400]  = SolarFluxPrediction( d.jD0, 'nominal' );
d.atm.aP                = aP(1); 
d.atm.f                 = f(1); 
d.atm.fHat              = fHat(1); 
d.atm.fHat400           = fHat400(1);
d.kWheels               = [];
d.inertiaRWA            = [];
d.tRWA                  = [];
d.dampingFun            = @RHSHysteresisDamper;
d.dampingData.Br        = 0.004; % T; remanence
d.dampingData.Bs        = 0.027; % T; saturation induction
d.dampingData.Hc        = 12;    % A/m; coercive force
d.dampingData.u         = [];    % rod unit vectors
d.dampingData.v         = [];    % rod volumes
